import ij.IJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.gui.GenericDialog;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;
import imagescience.images.ColorImage;
import imagescience.images.Coordinates;
import imagescience.images.Dimensions;
import imagescience.images.Image;
import imagescience.transforms.Embed;
import imagescience.utilities.I5DResource;
import imagescience.utilities.Messenger;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.image.ColorModel;

public class TJ_Embed implements PlugIn {

    private static int newxdim = 1024;
    private static int newydim = 1024;
    private static int newzdim = 1;
    private static int newtdim = 1;
    private static int newcdim = 1;

    private static int xpos = 0;
    private static int ypos = 0;
    private static int zpos = 1;
    private static int tpos = 1;
    private static int cpos = 1;
    
    private static final String[] fillings = { "zero", "minimum", "maximum", "repeat", "mirror", "clamp" };
    private static int filling = 0;

    public void run(String arg) {

	msg.debug(TJ_Options.debug);
	msg.debug("TransformJ "+TJ_Options.VERSION+": Embed");

	if (IJ.versionLessThan("1.35i")) return;

	final ImagePlus imp = WindowManager.getCurrentImage();
	if (imp == null)  { error("There are no images open"); return; }

	GenericDialog gd = new GenericDialog("TransformJ: Embed");
	gd.addNumericField("x-size of new image:",newxdim,0);
	gd.addNumericField("y-size of new image:",newydim,0);
	gd.addNumericField("z-size of new image:",newzdim,0);
	gd.addNumericField("t-size of new image:",newtdim,0);
	gd.addNumericField("ch-size of new image:",newcdim,0);
	gd.addPanel(new Panel(),GridBagConstraints.WEST,new Insets(6,0,0,0));
	gd.addNumericField("x-position of input image:",xpos,0);
	gd.addNumericField("y-position of input image:",ypos,0);
	gd.addNumericField("z-position of input image:",zpos,0);
	gd.addNumericField("t-position of input image:",tpos,0);
	gd.addNumericField("ch-position of input image:",cpos,0);
	gd.addPanel(new Panel(),GridBagConstraints.WEST,new Insets(0,0,0,0));
	gd.addChoice("Background filling:",fillings,fillings[filling]);
	gd.showDialog();

	if (gd.wasCanceled()) return;

	newxdim = (int)gd.getNextNumber();
	newydim = (int)gd.getNextNumber();
	newzdim = (int)gd.getNextNumber();
	newtdim = (int)gd.getNextNumber();
	newcdim = (int)gd.getNextNumber();
	xpos = (int)gd.getNextNumber();
	ypos = (int)gd.getNextNumber();
	zpos = (int)gd.getNextNumber();
	tpos = (int)gd.getNextNumber();
	cpos = (int)gd.getNextNumber();
	filling = gd.getNextChoiceIndex();

	try {
	    final Image img = Image.wrap(imp);
	    final Embed embedder = new Embed();
	    embedder.debug(TJ_Options.debug);
	    int filltype = Embed.ZERO;
	    switch (filling) {
	    case 0: filltype = Embed.ZERO; break;
	    case 1: filltype = Embed.MINIMUM; break;
	    case 2: filltype = Embed.MAXIMUM; break;
	    case 3: filltype = Embed.REPEAT; break;
	    case 4: filltype = Embed.MIRROR; break;
	    case 5: filltype = Embed.CLAMP; break;
	    }
	    if (newxdim < 1) throw new IllegalArgumentException("Zero or negative x-size for new image");
	    if (newydim < 1) throw new IllegalArgumentException("Zero or negative y-size for new image");
	    if (newzdim < 1) throw new IllegalArgumentException("Zero or negative z-size for new image");
	    if (newtdim < 1) throw new IllegalArgumentException("Zero or negative t-size for new image");
	    if (newcdim < 1) throw new IllegalArgumentException("Zero or negative ch-size for new image");
	    final Dimensions newdims = new Dimensions(newxdim,newydim,newzdim,newtdim,newcdim);
	    final Coordinates inpos = new Coordinates(xpos,ypos,zpos-1,tpos-1,cpos-1);
	    final Image newimg = embedder.run(img,newdims,inpos,filltype);
	    ImagePlus newimp = newimg.imageplus();
	    newimp.setCalibration(imp.getCalibration().copy());
	    final ImageProcessor ip = imp.getProcessor();
	    final ImageProcessor newip = newimp.getProcessor();
	    final ColorModel cm = ip.getColorModel();
	    newip.setColorModel(cm);
	    newimp.getStack().setColorModel(cm);
	    newip.setMinAndMax(ip.getMin(),ip.getMax());
	    boolean i5dinstalled = false;
	    try { Class.forName("Image5D"); i5dinstalled = true; } catch (Exception e) { }
	    if (i5dinstalled && !(newimg instanceof ColorImage)) {
		if (newdims.t > 1 || newdims.s > 1) {
		    if (I5DResource.instance(imp)) {
			newimp = I5DResource.convert(newimp,false);
			final int[][] idx = new int[2][];
			final Dimensions indims = img.dimensions();
			switch (filling) {
			case 0: case 1: case 2:
			    idx[0] = new int[indims.s]; idx[1] = new int[indims.s];
			    for (int i=0; i<indims.s; ++i) { idx[0][i] = i + 1; idx[1][i] = inpos.s + i + 1; }
			    break;
			case 3:
			    idx[0] = new int[newdims.s]; idx[1] = new int[newdims.s];
			    for (int i=0; i<indims.s; ++i) idx[0][inpos.s+i] = i + 1;
			    for (int i=0; i<newdims.s; ++i) idx[1][i] = i + 1;
			    for (int i=inpos.s-1, i0=inpos.s+indims.s-1; i>=0; --i, --i0) idx[0][i] = idx[0][i0];
			    for (int i=inpos.s+indims.s, i0=inpos.s; i<newdims.s; ++i, ++i0) idx[0][i] = idx[0][i0];
			    break;
			case 4:
			    idx[0] = new int[newdims.s]; idx[1] = new int[newdims.s];
			    for (int i=0; i<indims.s; ++i) idx[0][inpos.s+i] = i + 1;
			    for (int i=0; i<newdims.s; ++i) idx[1][i] = i + 1;
			    int ifs = 2; int indimssm1 = indims.s - 1;
			    if (indims.s == 1) { ++indimssm1; ifs = 1; }
			    for (int i=inpos.s-1; i>=0; --i) {
				final int idiff = i - inpos.s;
				int i0 = idiff / indimssm1; i0 += i0 % ifs;
				idx[0][i] = idx[0][inpos.s + Math.abs(idiff - i0*indimssm1)]; }
			    for (int i=inpos.s+indims.s; i<newdims.s; ++i) {
				final int idiff = i - inpos.s;
				int i0 = idiff / indimssm1; i0 += i0 % ifs;
				idx[0][i] = idx[0][inpos.s + Math.abs(idiff - i0*indimssm1)]; }
			    break;
			case 5:
			    idx[0] = new int[newdims.s]; idx[1] = new int[newdims.s];
			    for (int i=0; i<indims.s; ++i) idx[0][inpos.s+i] = i + 1;
			    for (int i=0; i<newdims.s; ++i) idx[1][i] = i + 1;
			    final int b = idx[0][inpos.s];
			    final int e = idx[0][inpos.s + indims.s - 1];
			    for (int i=inpos.s-1; i>=0; --i) idx[0][i] = b;
			    for (int i=inpos.s+indims.s; i<newdims.s; ++i) idx[0][i] = e;
			    break;
			}
			I5DResource.transfer(imp,newimp,idx);
		    } else {
			newimp = I5DResource.convert(newimp,true);
		    }
		}
	    }
	    newimp.changes = TJ_Options.asksave;
	    newimp.show();

	} catch (OutOfMemoryError e) {
	    error("Not enough memory for this operation");

	} catch (IllegalArgumentException e) {
	    error(e.getMessage());
	}
    }

    private void error(final String message) { msg.error("TransformJ: Error", message+"."); }

    private final Messenger msg = new Messenger();

}
